#!/bin/bash -u
#!/bin/ksh
#!/bin/sh
#-------------------------------------------------------------------------------
# PROGRAMM.sh                                    (C) 2024 T.Birnthaler OSTC GmbH
# BESCHREIBUNG
# $Id:$
#-------------------------------------------------------------------------------
# Skript autom. mit "root"-Rechten neu aufrufen (oder abbrechen) falls nicht so:
[ $(id -u) != 0 ] && echo "you have to be root!" 1>&2 && exec sudo $0 ${1+"$@"}
[ $(id -u) != 0 ] && echo "you have to be root!" 1>&2 && exit 1

# Sinnvolle Variablen-Initialisierungen am Programmanfang
DEBUG="false"                        # Debugging deaktivieren
IFS=" 	
"                                    # Standard-Shell-Trennzeichen (LEERZ, TAB, CR)
PATH="/bin:/usr/bin"                 # Normaler Suchpfad: Programme nur hier suchen
PATH="/bin:/usr/bin:/sbin:/usr/sbin" # root-Suchpfad: Programme nur hier suchen
LS_OPTIONS=                          # "ls"-Optionen aus
GROUP="users"                        # Standard-Gruppe
WORKDIR="/home/tsbirn"               # Arbeitsverzeichnis
EDITOR=                              # Standard-Editor
VISUAL=                              # Standard-Editor
LANG="C"                             # Standard-Sprache (wegen Meldungen!)
LANGUAGE="C"                         # Standard-Sprache (wegen Meldungen!)
LC_ALL="C"                           # Standard-Sprache (wegen Meldungen!)
SHELL="/bin/sh"                      # Standard-Shell
SHELL="/bin/ksh"                     # Standard-Shell
SHELL="/bin/bash"                    # Standard-Shell
LD_LIBRARY_PATH="/lib:/usr/lib"      # Standard-Bibliotheken
DBG=                                 # Debugging aus
TMP="/tmp"                           # Temporaeres Verz. setzen
TMPDIR="/tmp"                        # Temporaeres Verz. setzen

# Betriebssystem-Typ erkennen
if [[ "$OSTYPE" =~ darwin ]]; then
	echo "OSX"
elif [[ "$OSTYPE" =~ linux ]]; then
	echo "Linux"
else
	echo "Unknown"
fi

# Hilfsfunktionen fuer Ausgabe
function Error()   { FMT="ERROR: $1"; shift; printf "$FMT" ${1+"$@"} 1>&2; }
function Message() { $QUIET && return;
                     FMT="$1";        shift; printf "$FMT" ${1+"$@"} 1>&2; }
function Debug()   { $DEBUG || return;
                     FMT="DEBUG: $1"; shift; printf "$FMT" ${1+"$@"} 1>&2; }

# Usage-Meldung (evtl. wird eine Fehler-Meldung in "$1" uebergeben)
Usage()
{
	exec 1>&2                        # Alle Ausgaben auf Fehlerkanal umlenken

	[ -n "$1" ] && Error ${1+"$@"}
	echo "USAGE: $(basename $0) [OPTIONS]"
	echo "       -d      Debugging aktivieren [debug]"
	echo "       -h      Usage-Meldung erzeugen [help]"
	echo "       -q      Meldungen unterdruecken [quiet]"
	echo "       -v VAL  Wert VAL eintragen [value]"

	exit 1
}

# Optionsvariable leeren
DEBUG=false
QUIET=false
VALUE=

# Bekannte Optionen erkennen, falsche abfangen
while getopts ":dhqv:" OPT   # ":" am Anfang unterdrueckt Fehlermeldung
do                           # ":" nach Buchstabe heisst "Option mit Parameter"
	case "$OPT" in
		d) DEBUG="true" ;;
		h) Usage "" ;;
		q) QUIET="true" ;;
		v) VALUE="$OPTARG" ;;
		*) Error "FEHLER: Unbekannte Option -$OPTARG" ;;
	esac
done

# Optionen aus Parameterliste rauswerfen (WICHTIG!)
shift $(expr $OPTIND - 1)

# Optionen anzeigen
echo "Option -d: $DEBUG"
echo "Option -q: $QUIET"
echo "Option -v: $VALUE"

# Restliche Argumente anzeigen
echo "Argumente($#): $*"

# Restliche Argumente auflisten
for ARG
do
	echo "ARG: $ARG"
done

# Gruppe + alle Anderen auf im Skript erz. Dateien/Verz. nicht zugreifen lassen
umask 077

# Namen von temp. Dateien/Verz. nicht erratbar generieren und anlegen
 TMPDIR=$(mktemp -d /tmp/secshXXXXXXXXXXX) || exit 1 # Temp. Verz. (mind. 12 X!)
TMPFILE=$(mktemp $TMPDIR/secshXXXXXXXXXXX) || exit 1 # Temp. Datei (mind. 12 X!)

# Gewuenschter Benutzer(-Gruppe) zuordnen + Abbruch wenn nicht moeglich
chgrp $GROUP $TMPDIR $TMPFILE || exit 1

# Wichtige Signale abfangen
trap "exit 1" 1   2   3    13   14   15   17   # --> Signal 0 = Skript-Ende ausloesen
trap "exit 1" HUP INT QUIT PIPE ALRM TERM CHLD # --> Signal 0 = Skript-Ende ausloesen

# Temporaere Dateien/Verz. bei Skriptende (Signal 0/EXIT) automatisch loeschen
trap 'rm -rf $TMPFILE $TMPDIR > /dev/null 2>&1' EXIT
trap 'rm -rf $TMPFILE $TMPDIR > /dev/null 2>&1' 0

# WICHTIG: Mit "cd" in ein Verz. wechseln
# --> IMMER Fehlermeldung + ABBRUCH wenn NICHT moeglich!
cd "$WORKDIR" > /dev/null ||
	{ echo "ERROR: Kann nicht in Verz. '$WORKDIR' wechseln" > 1>&2; exit 1 }

# Debugging anschalten?
[ "$1" = "-d" ] && DBG=1

# Wertzuweisung und Variablenverwendung immer mit "..." aussenrum
# (leer oder Metazeichen --> Peng!)
VAR="Wert"
echo "$VAR"

# Vergleiche immer gegen leere Variablen UND Optionen schuetzen
if [ "$VAR" = "abc" ]...         # Leerraum und Sonderzeichen schuetzen
if [ "X$VAR" = "Xabc" ]...       # Falls Variablenwert mit "-" beginnt!

# Dateilisten mit "/dev/null" abschliessen
# (vermeidet leere Liste --> Warten auf Benutzereingabe)
cat * /dev/null
cat $* /dev/null

# Parameterliste exakt reproduzieren ($*, $@, "$*" und "$@" stimmen nicht ganz)
subskript.sh ${1+"$@"}
