#!/usr/bin/mysql -utom -pcat -e
#-------------------------------------------------------------------------------
# PROGRAMM.sql                                   (C) 2024 T.Birnthaler OSTC GmbH
# BESCHREIBUNG
# $Id:$
#-------------------------------------------------------------------------------
DROP DATABASE IF EXISTS test;
CREATE DATABASE IF NOT EXISTS test;

SHOW DATABASES;

USE test;

DROP TABLE If EXISTS test;
CREATE TABLE IF NOT EXISTS test (
    nr        INT NOT NULL AUTO_INCREMENT,        # Automatisch fuellen
    anz       TINYINT(3) UNSIGNED,                # Ausgabelaenge 3
    grad      FLOAT(7,1),                         # Ausgabeformat 12345.6
    betrag    DECIMAL(9,2),                       # Ausgabeformat 123456.78
    vorname   CHAR(30) NOT NULL,                  # Zeichen (mit Leerz. aufgef.)
    name      VARCHAR(30),                        # Zeichen
    binaer1   BINARY(10),                         # Byte (mit 0-Byte aufgefuellt)
    binaer2   VARBINARY(8),                       # Byte
    stamp1    TIMESTAMP,                          # Automatisch gefuellt (1.Sp.)
    stamp2    TIMESTAMP,                          # NICHT autom. gefuellt (2.Sp.)
    stamp3    TIMESTAMP,                          # NICHT autom. gefuellt (3.Sp.)
    datum     DATE DEFAULT '2000-01-01',          #
    uhrzeit   TIME DEFAULT '12:00:00',            #
    zeitpunkt DATETIME,                           # JJJJ-MM-TT hh:mm:ss
    jahr1     YEAR(4),                            #
    jahr2     YEAR(2),                            #
    opt       BOOL,                               # TRUE/FALSE
    flags     BIT(8),                             # Bitmuster 8 Bit = 1 Byte
    dokument  TEXT,                               #
    antwort   ENUM('Ja', 'Nein', 'Vielleicht'),   # Wert aus Liste
    menge     SET('gross','reich','maechtig'),    # Menge aus Listenwerten
    file      BLOB,
    PRIMARY KEY (nr)   # Kein Komma, notwendig wg. AUTO_INCREMENT von Spalte nr
) ENGINE = "InnoDB";

SHOW TABLES;
DESC test;

INSERT INTO test VALUES (
    NULL,                     # INT:          Naechste freie Nr. wg. AUTO_INCREMENT
    123,                      # TINYINT(3):   fuellen
    12345.6,                  # FLOAT(7,1):   fuellen
    876543.21,                # DECIMAL(9,2): fuellen
    "Thomas",                 # CHAR(30):     fuellen (mit Leerz. hinten auffuellen)
    "Birnthaler",             # VARCHAR(30):  fuellen
    0x4142434445464748494A,   # BINARY(10):   fuellen (mit 0-Byte hinten auffuellen)
    x'6162636465666768',      # VARBINARY(8): fuellen
    NULL,                     # TIMESTAMP:    Aktuellen Zeitp. autom. eintragen
    "20240907112233",         # TIMESTAMP:    fuellen
    "2024-09-07 11:22:33",    # TIMESTAMP:    fuellen ("7.9.2024 11:22:33" NICHT ok!)
    "2024-08-03",             # DATE:         fuellen
    "23:59:59",               # TIME:         fuellen
    "2024-08-03 23:59:59",    # DATETIME:     fuellen ("3.8.2024 23:59:59" NICHT ok!)
    2024,                     # YEAR(4):      fuellen
    2024,                     # YEAR(2):      fuellen
    TRUE,                     # BOOL:         fuellen
    b'01000001',              # BIT(8):       fuellen ASCII-Code 65 = Zeichen 'A'
    "Dokument(text)",         # TEXT:         fuellen (direkt) oder
  # LOAD_FILE("/etc/my.cnf"), # TEXT:                aus externer Datei laden
    "Vielleicht",             # ENUM:         Aufzaehlungswert als String (EINER!)
    "gross,maechtig",         # SET:          Elem.-Liste per "," getrennt als String
    "Blobdaten(binaer)"       # BLOB:         fuellen (direkt) oder
  # LOAD_FILE("/etc/my.cnf")  # BLOB:                 aus externer Datei laden
  # 0x4142434445464748494A,   # BLOB:         fuellen (mit 0-Byte hinten auffuellen)
  # x'6162636465666768',      # BLOB:         fuellen
);

SELECT * FROM test;

CREATE USER tom;                 # Leeres Passwort, keine Zugriffsrechte

GRANT ALL PRIVILEGES             # Alle Zugriffsrechte...
  ON test.*                      # ...auf Objekte der Datenbank "first"
    TO "tom"@"localhost"         # ...fuer Benutzer "tom@localhost"
      IDENTIFIED BY "geheim"     # ...mit Passwort "geheim"
        WITH GRANT OPTION;       # ...mit Rechteweitergabe (optional)

SET PASSWORD FOR "tom"@"localhost" = PASSWORD("geheim");

